; 
; This program displays the System Memory Map returned by
; Interrupt 15h Function E820h along with the calculated
; total Extended Memory size, the Extended Memory size as
; returned by Interrupt 15h Function 88h, and the Extended
; Memory size as read from the CMOS RAM.
;
; Note that Interrupt 15h, Function E820h is supported on
; "recent" systems only, where "recent" probably means
; 2000+.
;  
; Note that Interrupt 15h, Function E820h is not supported
; under Windows 2000/XP. Note also that the KB of Extended
; Memory as returned by Interrupt 15h Function 88h, and as
; stored in the CMOS RAM, is a 16-bit value that cannot
; represent more than (65535*1024) bytes (~64 MB).
; 
      .model small,c
      .386
      .stack

      include support.asm      

      SMM_ADDRESS_RANGE_DESC STRUCT
         baseLo      dd 0
         baseHi      dd 0
         lengthLo    dd 0
         lengthHi    dd 0
         rangeType   dd 0
      SMM_ADDRESS_RANGE_DESC ENDS
      RT_AVAILABLE      EQU 1
      RT_RESERVED       EQU 2
      RT_ACPI_RECLAIM   EQU 3
      RT_ACPI_NVS       EQU 4

.data
      smmard SMM_ADDRESS_RANGE_DESC <>

      memTotal    dd 0
.code
.startup
; 
      loc 1,0
      
      print "Memory Map returned by Interrupt 15h Function E820h:",NL

      ; Set ES to data segment.
      push  ds
      pop   es
      ; Load the starting value (zero) into EBX.
      xor   ebx,ebx
   callLoop:
      mov   eax,0e820h
      mov   edx,'SMAP'
      mov   ecx,SIZEOF smmard
      ; Set DI to the offset address of smmard.
      lea   di,smmard
      int   15h
      cmp   eax,'SMAP'
      je    @F
      print NL,"* Interrupt 15h Function E820h is not supported *",NL,NL
      jmp   callFunction88h      
   @@:
      ; The function may indicate that the last range
      ; has been returned by setting the carry flag
      ; or by returning zero in EBX.
      jc    finish
      test  ebx,ebx
      jz    finish   

      invoke dw2hex,smmard.baseLo,ADDR outbuf
      print NL,"BASE=",ADDR outbuf,"h "
      invoke dw2hex,smmard.lengthLo,ADDR outbuf
      print "LENGTH=",ADDR outbuf,"h "
      invoke dw2dec,smmard.lengthLo,ADDR outbuf
      print chr$(40),ADDR outbuf,chr$(41)
      loc ,44
      print "TYPE="
      .IF smmard.rangeType == RT_AVAILABLE
         print "AVAILABLE"
      .ELSEIF smmard.rangeType == RT_RESERVED
         print "RESERVED"
      .ELSEIF smmard.rangeType == RT_ACPI_RECLAIM
         print "ACPI_RECLAIM"
      .ELSEIF smmard.rangeType == RT_ACPI_NVS
         print "ACPI_NVS"
      .ENDIF

      ; Update the total length.
      mov   eax,smmard.lengthLo
      add   memTotal,eax

      jmp   callLoop

   finish:

      print NL,"------------------------------------------------------------"
      print NL,"TOTAL LENGTH          "
      invoke dw2hex,memTotal,ADDR outbuf
      print ADDR outbuf
      invoke dw2dec,memTotal,ADDR outbuf
      print "h ",chr$(40),ADDR outbuf,chr$(41),NL,NL

   callFunction88h:
      
      mov   ah,88h
      int   15h
      jnc   @F
      print "Interrupt 15h Function 88h returned an error",NL,NL
      jmp   getFromCMOS
   @@:
      ; Zero extend the return value into EAX and
      ; multiply by 1024.
      movzx eax,ax
      mov   edx,1024
      mul   edx
      invoke dw2dec,eax,ADDR outbuf
      print "Extended Memory size returned by Interrupt 15h Function 88h = "
      print ADDR outbuf," bytes",NL,NL

   getFromCMOS:      
      ; Get the Extended-Memory-in-KB-low-byte and
      ; store it in DL.
      invoke read_cmos,30h
      mov   dl,al
      ; Get the Extended-Memory-in-KB-high-byte and
      ; store it in DH.
      invoke read_cmos,31h
      mov   dh,al
      ; Zero extend the word value into EDX and
      ; multiply by 1024.
      movzx edx,dx
      mov   eax,1024
      mul   edx
      invoke dw2dec,eax,ADDR outbuf
      print "Extended Memory size from CMOS RAM = "
      print ADDR outbuf," bytes",NL,NL

      print NL,"Press any normal key to exit...",NL
      call waitkey
; 
.exit

end
