; 
; This file contains a (incomplete) collection of MASM 6+
; macros, equates, and data and procedure definitions aimed
; at supporting real-mode DOS programming.
; 

      ; DOS handles for standard input/output devices.
      HSTDIN   EQU 0
      HSTDOUT  EQU 1

      stdin       PROTO C :WORD,:WORD
      stdout      PROTO C :WORD
      szlen       PROTO C :WORD
      locate      PROTO c :BYTE,:BYTE
      w2dec       PROTO C :WORD,:WORD
      w2hex       PROTO C :WORD,:WORD
      dw2dec      PROTO c :DWORD,:WORD
      dw2hex      PROTO c :DWORD,:WORD
      b2bin       PROTO c :BYTE,:WORD
      read_cmos   PROTO c :BYTE

; 

      ; This is the MASM32 reparg macro with *no* changes.
      ;
      ; "This macro replaces quoted text with a DATA section
      ; OFFSET and returns it in ADDR "name" format."
      ;
      reparg MACRO arg
         LOCAL nustr
         quot SUBSTR <arg>,1,1
         IFIDN quot,<">       ;; if 1st char = "
         .data
            nustr db arg,0    ;; write arg to .DATA section
         .code
         EXITM <ADDR nustr>   ;; append name to ADDR operator
         ELSE
            EXITM <arg>       ;; else return arg
         ENDIF
      ENDM

      ; This is a VARARG version of the MASM32 print macro.
      ; It will accept either a quoted string, or the offset
      ; address of a null-terminated string in the data
      ; segment
      ;
      print MACRO varname:VARARG
         FOR arg,<varname>
            invoke stdout,reparg(arg)
         ENDM
      ENDM

      ; This is the MASM32 chr$ macro with *no* changes.
      ;
      ; "This macro takes quoted string and ascii numbers in
      ; the same format as a .DATA section entry. It returns
      ; the offset in the .DATA section of the data contained
      ; in its parameters."
      ;
      chr$ MACRO any_text:VARARG
        LOCAL txtname
        .data
            txtname db any_text,0
        .code
        EXITM <OFFSET txtname>
      ENDM

      ; This macro sets the cursor position for display
      ; page 0 to the specified row and column coordinates.
      ; The unusual feature is that either or both of the
      ; coordinate parameters may be left blank, causing
      ; the cursor position with respect to the blank
      ; coordinate to remain unchanged. For example,
      ; "loc 5" would position the cursor at row 5 without
      ; changing the column position, and "loc ,5" would
      ; position the cursor at column 5 without changing
      ; the row position.
      ;
      loc MACRO row,col
         IFNB <row>
            IFNB <col>   
               invoke locate,row,col
            ELSE
               call cursorcol
               invoke locate,row,al
            ENDIF
         ELSE
            IFNB <col>   
               call cursorrow
               invoke locate,al,col
            ELSE
               call cursorrow
               mov   ah,al
               call cursorcol
               invoke locate,ah,al
            ENDIF
         ENDIF
      ENDM

; 
.data
      inbuf       db 130 dup(0)
      outbuf      db 30 dup(0)
      crlf        db 13,10,0
      NL          dw offset crlf
.code

; 
; This proc calls the Read File or Device function to read
; up to <blen> bytes from STDIN and store them in the
; specified buffer.
; 
stdin proc uses bx cx dx npdest:WORD,blen:WORD
      mov   bx,HSTDIN
      mov   cx,blen
      mov   dx,npdest
      mov   ah,3fh
      int   21h
      ; Replace CR with null.
      mov   bx,npdest
   @@:
      cmp   byte ptr[bx],0
      je    @F
      cmp   byte ptr[bx],13
      je    @F
      inc   bx
      jmp   @B
   @@:
      mov   byte ptr[bx],0
      ret
stdin endp

; 
; This proc calls the Write File or Device function to
; output the specified null-terminated string to STDOUT.
; 
stdout proc uses ax bx cx dx npsz:WORD
      invoke szlen,npsz
      mov   cx,ax
      mov   bx,HSTDOUT
      mov   dx,npsz
      mov   ah,40h
      int   21h
      ret
stdout endp

; 
; This proc returns the length of the specified
; null-terminated string in AX.
; 
szlen proc uses bx npsz:WORD
      mov   bx,npsz
   runLoop:
      ; Compare bytes until find null.
      cmp   byte ptr[bx],0
      jz    @F
      inc   bx
      jmp   runLoop
   @@:
      ; Calc length and return in AX.
      mov   ax,bx
      sub   ax,npsz
      ret
szlen endp

; 
; This proc calls the Flush Buffer, Read Keyboard Without
; Echo function and returns the character read in AL.
; 
waitkey proc
      mov   ax,0c08h
      int   21h
      ret
waitkey endp

; 
; This proc calls the BIOS Set Cursor Position function
; to set the cursor position for display page 0 to
; <row>,<col> (upper left corner is 0,0).
; 
locate proc uses ax bx dx row:BYTE,col:BYTE
      ; Function number.
      mov   ah,2
      ; Display page.
      mov   bh,0
      mov   dh,row
      mov   dl,col
      int   10h
      ret
locate endp

; 
; These procs return the row and column cursor positions
; for display page 0 in AL.
; 
cursorrow proc uses es
      ; Set ES to the BIOS data area.
      push  40h
      pop   es
      ; Read the appropriate byte.
      mov   al,es:[51h]
      ret
cursorrow endp
cursorcol proc uses es
      push  40h
      pop   es
      mov   al,es:[50h]
      ret
cursorcol endp

; 
; This proc clears the screen by calling the BIOS Scroll
; Current Page Up function with a scroll (row) count of zero,
; and sets the cursor position to the upper left corner of
; the screen. An 80x25 text mode screen is assumed.
; 
cls proc uses ax bx cx dx
      ; Function number.
      mov   ah,6
      ; Row count.
      mov   al,0
      ; Attribute for blanked lines (white on black).
      mov   bh,7
      ; Top row of window to scroll.
      mov   ch,0
      ; Left-most column of window to scroll.
      mov   cl,0
      ; Bottom row of window to scroll.
      mov   dh,24
      ; Right-most column of window to scroll.
      mov   dl,79
      int   10h
      invoke locate,0,0
      ret
cls endp

; 
; This proc converts a word to to a signed null-terminated
; decimal string and stores the result (6 characters max)
; in the specified buffer.
; 
w2dec proc uses ax bx cx dx di src:WORD,npdest:WORD
      mov   ax,src
      mov   di,npdest

      ; Test for negative value.
      test  ax,ax
      jns   @F
      ; Make positive if negative.
      neg   ax   
      ; Store sign in buffer.
      mov   byte ptr[di],'-'
      inc   di
   @@:
      ; Init divisor.
      mov   bx,10
      ; Zero digit counter.
      mov   cx,0
   divLoop:
      ; Zero MSW of dividend.
      xor   dx,dx      
      ; Divide value by 10.
      div   bx
      ; Push remainder.
      push  dx
      ; Increment digit count.
      inc   cx
      ; Repeat if quotient > 0.
      or    ax,ax
      jnz   divLoop
   storeLoop:
      ; Pop digit off stack.
      pop   ax
      ; Convert to ascii.
      add   al,'0'
      ; Store in buffer.
      mov   [di],al
      inc   di
      ; Repeat CX times.
      loop  storeLoop
      ; Append trailing null.
      mov   byte ptr[di],0
      ret
w2dec endp

; 
; This proc converts a word to to a null-terminated 4-character
; hex string and stores the result in the specified buffer.
; 
   .data
      hexChars db "0123456789ABCDEF"
   .code
w2hex proc uses ax bx dx di src:WORD,npdest:WORD
      mov   dx,src
      mov   di,npdest
      ; Set DI to end of buffer.
      add   di,3
      ; Append trailing null.
      mov   byte ptr[di+1],0
   @@:
      ; Copy current value into BX.
      mov   bx,dx
      ; Mask off all but lower nibble.
      and   bx,0Fh
      ; Load corresponding hex character into AL.
      mov   al,hexChars[bx]
      ; And store it in buffer.
      mov   [di],al
      ; Shift next nibble into position.
      shr   dx,4
      ; Adjust DI for next character.
      dec   di
      ; Repeat for all 4 characters.
      cmp   di,npdest
      jnb   @B
      ret
w2hex endp

; 
; This proc converts a dword to to an signed null-terminated
; decimal string and stores the result (11 characters max)
; in the specified buffer.
; 
dw2dec proc uses ax bx cx dx di src:DWORD,npdest:WORD
      mov   eax,src
      mov   di,npdest

      ; Test for negative value.
      test  eax,eax
      jns   @F
      ; Make positive if negative.
      neg   eax   
      ; Store sign in buffer.
      mov   byte ptr[di],'-'
      inc   di
   @@:
      ; Init divisor.
      mov   ebx,10
      ; Zero digit counter.
      mov   cx,0
   divLoop:
      ; Zero MSDW of dividend.
      xor   edx,edx
      ; Divide value by 10.
      div   ebx
      ; Push remainder.
      push  dx
      ; Increment digit count.
      inc   cx
      ; Repeat if quotient > 0.
      or    eax,eax
      jnz   divLoop
   storeLoop:
      ; Pop digit off stack.
      pop   ax
      ; Convert to ascii.
      add   al,'0'
      ; Store in buffer.
      mov   [di],al
      inc   di
      ; Repeat CX times.
      loop  storeLoop
      ; Append trailing null.
      mov   byte ptr[di],0
      ret
dw2dec endp

; 
; This proc converts a dword to to a null-terminated
; 8-character hex string and stores the result in the
; specified buffer.
;  
dw2hex proc uses ax bx dx di src:DWORD,npdest:WORD
      mov   edx,src
      mov   di,npdest
      ; Set DI to end of buffer.
      add   di,7
      ; Append trailing null.
      mov   byte ptr[di+1],0
   @@:
      ; Copy current value into EBX.
      mov   ebx,edx
      ; Mask off all but lower nibble.
      and   ebx,0Fh
      ; Load corresponding hex character into AL.
      mov   al,hexChars[ebx]
      ; And store it in buffer.
      mov   [di],al
      ; Shift next nibble into position.
      shr   edx,4
      ; Adjust DI for next character.
      dec   di
      ; Repeat for all 8 characters.
      cmp   di,npdest
      jnb   @B
      ret
dw2hex endp

; 
; This proc converts a byte to to a null-terminated
; 8-character binary string and stores the result in
; the specified buffer.
; 
b2bin proc uses ax bx cx dx src:BYTE,npdest:WORD
      mov   al,src
      mov   bx,npdest
      mov   cx,8
      dec   bx
   shiftLoop:
      inc   bx
      xor   dx,dx
      shl   al,1
      adc   dl,'0'      
      dec   cx      
      mov   [bx],dl
      jnz   shiftLoop
      mov   byte ptr[bx+1],0
      ret
b2bin endp

; 
; This proc is essentially the READ_CMOS example from Frank
; van Gilluwe's book The Undocumented PC. It returns the
; value from the specified CMOS address in AL.
;
; Bit 7 of the CMOS address port is used to control the
; state of the NMI (Nonmaskable Interrupt). The remaining
; 7 bits are used to specify one of 128 possible addresses.
;
; Note that this procedure does not implement the logic
; necessary to reliably read the time and date registers.
;
; In practice, for anything resembling a recent system,
; the I/O delay is not necessary because the chipset
; inserts its own delay.
; 
IODELAY MACRO      
      jmp   $+2
      jmp   $+2
ENDM
read_cmos proc address:BYTE
      mov   al,address
      ; Set bit 7 of the address to disable the NMI.
      or    al,80h
      ; Disable hardware interrupts.
      cli
      ; Write the address to the CMOS address port.
      out   70h,al
      IODELAY
      ; Read the addressed value from the CMOS data port.
      in    al,71h
      IODELAY
      ; Preserve the value.
      mov   ah,al
      ; Write a zero to the address port to enable the NMI.      
      mov   al,0
      out   70h,al
      ; Shift the value into AL, forcing AH to zero.
      shr   ax,8
      ; Enable hardware interrupts.
      sti
      ret
read_cmos endp
